@@ -5,6 +5,7 @@ from scipy.ndimage.filters import convolve
 from scipy.special import gamma
 
 from basicsr.metrics.metric_util import reorder_image, to_y_channel
+from basicsr.utils.matlab_functions import imresize
 
 
 def estimate_aggd_param(block):
@@ -114,12 +115,9 @@ def niqe(img, mu_pris_param, cov_pris_param, gaussian_window, block_size_h=96, b
                 feat.append(compute_feature(block))
 
         distparam.append(np.array(feat))
-        # TODO: matlab bicubic downsample with anti-aliasing
-        # for simplicity, now we use opencv instead, which will result in
-        # a slight difference.
+
         if scale == 1:
-            h, w = img.shape
-            img = cv2.resize(img / 255., (w // 2, h // 2), interpolation=cv2.INTER_LINEAR)
+            img = imresize(img / 255., scale=0.5, antialiasing=True)
             img = img * 255.
 
     distparam = np.concatenate(distparam, axis=1)
@@ -134,8 +132,9 @@ def niqe(img, mu_pris_param, cov_pris_param, gaussian_window, block_size_h=96, b
     invcov_param = np.linalg.pinv((cov_pris_param + cov_distparam) / 2)
     quality = np.matmul(
         np.matmul((mu_pris_param - mu_distparam), invcov_param), np.transpose((mu_pris_param - mu_distparam)))
-    quality = np.sqrt(quality)
 
+    quality = np.sqrt(quality)
+    quality = np.squeeze(quality)
     return quality
 
 
@@ -146,6 +145,9 @@ def calculate_niqe(img, crop_border, input_order='HWC', convert_to='y'):
     This implementation could produce almost the same results as the official
     MATLAB codes: http://live.ece.utexas.edu/research/quality/niqe_release.zip
 
+    > MATLAB R2021a result for tests/data/baboon.png: 5.72957338 (5.7296)
+    > Our re-implementation result for tests/data/baboon.png: 5.7295763 (5.7296)
+
     We use the official params estimated from the pristine dataset.
     We use the recommended block size (96, 96) without overlaps.
 
@@ -184,6 +186,9 @@ def calculate_niqe(img, crop_border, input_order='HWC', convert_to='y'):
     if crop_border != 0:
         img = img[crop_border:-crop_border, crop_border:-crop_border]
 
+    # round is necessary for being consistent with MATLAB's result
+    img = img.round()
+
     niqe_result = niqe(img, mu_pris_param, cov_pris_param, gaussian_window)
 
     return niqe_result
